package sun.awt.windows;

import sun.java2d.pipe.Region;
import sun.awt.Win32GraphicsEnvironment;
import java.awt.peer.ContainerPeer;
import java.awt.AWTException;
import java.awt.dnd.DropTarget;
import sun.awt.AWTAccessor;
import java.awt.Toolkit;
import java.awt.image.ImageObserver;
import sun.awt.image.SunVolatileImage;
import sun.awt.image.ToolkitImage;
import java.awt.Image;
import java.awt.image.ImageProducer;
import java.awt.FontMetrics;
import java.awt.Window;
import java.awt.SystemColor;
import java.awt.image.ColorModel;
import java.awt.event.InvocationEvent;
import sun.awt.PaintEventDispatcher;
import sun.java2d.ScreenUpdateManager;
import java.awt.GraphicsConfiguration;
import java.awt.event.FocusEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.InputEvent;
import java.awt.AWTEvent;
import sun.awt.KeyboardFocusManagerPeerImpl;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.Rectangle;
import sun.awt.event.IgnorePaintEvent;
import java.awt.event.PaintEvent;
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import sun.java2d.opengl.OGLSurfaceData;
import sun.java2d.d3d.D3DSurfaceData;
import java.awt.Container;
import sun.awt.SunToolkit;
import sun.java2d.InvalidPipeException;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Point;
import java.awt.Font;
import java.awt.Color;
import java.awt.BufferCapabilities;
import java.awt.image.VolatileImage;
import sun.awt.Win32GraphicsConfig;
import sun.awt.RepaintArea;
import sun.java2d.SurfaceData;
import sun.util.logging.PlatformLogger;
import java.awt.dnd.peer.DropTargetPeer;
import java.awt.peer.ComponentPeer;

public abstract class WComponentPeer extends WObjectPeer implements ComponentPeer, DropTargetPeer
{
    protected volatile long hwnd;
    private static final PlatformLogger log;
    private static final PlatformLogger shapeLog;
    private static final PlatformLogger focusLog;
    SurfaceData surfaceData;
    private RepaintArea paintArea;
    protected Win32GraphicsConfig winGraphicsConfig;
    boolean isLayouting;
    boolean paintPending;
    int oldWidth;
    int oldHeight;
    private int numBackBuffers;
    private VolatileImage backBuffer;
    private BufferCapabilities backBufferCaps;
    private Color foreground;
    private Color background;
    private Font font;
    int nDropTargets;
    long nativeDropTargetContext;
    public int serialNum;
    private static final double BANDING_DIVISOR = 4.0;
    static final Font defaultFont;
    private int updateX1;
    private int updateY1;
    private int updateX2;
    private int updateY2;
    private volatile boolean isAccelCapable;
    
    @Override
    public native boolean isObscured();
    
    @Override
    public boolean canDetermineObscurity() {
        return true;
    }
    
    private synchronized native void pShow();
    
    synchronized native void hide();
    
    synchronized native void enable();
    
    synchronized native void disable();
    
    public long getHWnd() {
        return this.hwnd;
    }
    
    @Override
    public native Point getLocationOnScreen();
    
    @Override
    public void setVisible(boolean b) {
        if (b) {
            this.show();
        }
        else {
            this.hide();
        }
    }
    
    public void show() {
        Dimension size = ((Component)this.target).getSize();
        this.oldHeight = size.height;
        this.oldWidth = size.width;
        this.pShow();
    }
    
    @Override
    public void setEnabled(boolean b) {
        if (b) {
            this.enable();
        }
        else {
            this.disable();
        }
    }
    
    private native void reshapeNoCheck(int p0, int p1, int p2, int p3);
    
    @Override
    public void setBounds(int n, int n2, int oldWidth, int oldHeight, int n3) {
        this.paintPending = (oldWidth != this.oldWidth || oldHeight != this.oldHeight);
        if ((n3 & 0x4000) != 0x0) {
            this.reshapeNoCheck(n, n2, oldWidth, oldHeight);
        }
        else {
            this.reshape(n, n2, oldWidth, oldHeight);
        }
        if (oldWidth != this.oldWidth || oldHeight != this.oldHeight) {
            try {
                this.replaceSurfaceData();
            }
            catch (InvalidPipeException ex) {}
            this.oldWidth = oldWidth;
            this.oldHeight = oldHeight;
        }
        ++this.serialNum;
    }
    
    void dynamicallyLayoutContainer() {
        if (WComponentPeer.log.isLoggable(PlatformLogger.Level.FINE) && SunToolkit.getNativeContainer((Component)this.target) != null) {
            WComponentPeer.log.fine("Assertion (parent == null) failed");
        }
        final Container target = (Container)this.target;
        SunToolkit.executeOnEventHandlerThread(target, new Runnable() {
            @Override
            public void run() {
                target.invalidate();
                target.validate();
                if (!(WComponentPeer.this.surfaceData instanceof D3DSurfaceData.D3DWindowSurfaceData) && !(WComponentPeer.this.surfaceData instanceof OGLSurfaceData)) {
                    return;
                }
                try {
                    WComponentPeer.this.replaceSurfaceData();
                }
                catch (InvalidPipeException ex) {}
            }
        });
    }
    
    void paintDamagedAreaImmediately() {
        this.updateWindow();
        SunToolkit.flushPendingEvents();
        this.paintArea.paint(this.target, this.shouldClearRectBeforePaint());
    }
    
    synchronized native void updateWindow();
    
    @Override
    public void paint(Graphics g) {
        ((Component)this.target).paint(g);
    }
    
    public void repaint(long n, int n2, int n3, int n4, int n5) {
    }
    
    private native int[] createPrintedPixels(int p0, int p1, int p2, int p3, int p4);
    
    @Override
    public void print(Graphics g) {
        Component component = (Component)this.target;
        int width = component.getWidth();
        int height = component.getHeight();
        int n = (int)((double)height / 4.0);
        if (n == 0) {
            n = height;
        }
        for (int i = 0; i < height; i += n) {
            int n2 = i + n - 1;
            if (n2 >= height) {
                n2 = height - 1;
            }
            int n3 = n2 - i + 1;
            Color background = component.getBackground();
            int[] printedPixels = this.createPrintedPixels(0, i, width, n3, (background == null) ? 255 : background.getAlpha());
            if (printedPixels != null) {
                BufferedImage o = new BufferedImage(width, n3, 2);
                o.setRGB(0, 0, width, n3, printedPixels, 0, width);
                g.drawImage(o, 0, i, null);
                o.flush();
            }
        }
        component.print(g);
    }
    
    @Override
    public void coalescePaintEvent(PaintEvent paintEvent) {
        Rectangle updateRect = paintEvent.getUpdateRect();
        if (!(paintEvent instanceof IgnorePaintEvent)) {
            this.paintArea.add(updateRect, paintEvent.getID());
        }
        if (WComponentPeer.log.isLoggable(PlatformLogger.Level.FINEST)) {
            switch (paintEvent.getID()) {
                case 801: {
                    WComponentPeer.log.finest("coalescePaintEvent: UPDATE: add: x = " + updateRect.x + ", y = " + updateRect.y + ", width = " + updateRect.width + ", height = " + updateRect.height);
                    return;
                }
                case 800: {
                    WComponentPeer.log.finest("coalescePaintEvent: PAINT: add: x = " + updateRect.x + ", y = " + updateRect.y + ", width = " + updateRect.width + ", height = " + updateRect.height);
                    return;
                }
                default: {
                    break;
                }
            }
        }
    }
    
    public synchronized native void reshape(int p0, int p1, int p2, int p3);
    
    public boolean handleJavaKeyEvent(KeyEvent keyEvent) {
        return false;
    }
    
    public void handleJavaMouseEvent(MouseEvent mouseEvent) {
        switch (mouseEvent.getID()) {
            case 501: {
                if (this.target == mouseEvent.getSource() && !((Component)this.target).isFocusOwner() && KeyboardFocusManagerPeerImpl.shouldFocusOnClick((Component)this.target)) {
                    WKeyboardFocusManagerPeer.requestFocusFor((Component)this.target, CausedFocusEvent.Cause.MOUSE_EVENT);
                    break;
                }
                break;
            }
        }
    }
    
    native void nativeHandleEvent(AWTEvent p0);
    
    @Override
    public void handleEvent(AWTEvent awtEvent) {
        int id = awtEvent.getID();
        if (awtEvent instanceof InputEvent && !((InputEvent)awtEvent).isConsumed() && ((Component)this.target).isEnabled()) {
            if (awtEvent instanceof MouseEvent && !(awtEvent instanceof MouseWheelEvent)) {
                this.handleJavaMouseEvent((MouseEvent)awtEvent);
            }
            else if (awtEvent instanceof KeyEvent && this.handleJavaKeyEvent((KeyEvent)awtEvent)) {
                return;
            }
        }
        switch (id) {
            case 800: {
                this.paintPending = false;
            }
            case 801: {
                if (!this.isLayouting && !this.paintPending) {
                    this.paintArea.paint(this.target, this.shouldClearRectBeforePaint());
                }
                return;
            }
            case 1004:
            case 1005: {
                this.handleJavaFocusEvent((FocusEvent)awtEvent);
                break;
            }
        }
        this.nativeHandleEvent(awtEvent);
    }
    
    void handleJavaFocusEvent(FocusEvent focusEvent) {
        if (WComponentPeer.focusLog.isLoggable(PlatformLogger.Level.FINER)) {
            WComponentPeer.focusLog.finer(focusEvent.toString());
        }
        this.setFocus(focusEvent.getID() == 1004);
    }
    
    native void setFocus(boolean p0);
    
    @Override
    public Dimension getMinimumSize() {
        return ((Component)this.target).getSize();
    }
    
    @Override
    public Dimension getPreferredSize() {
        return this.getMinimumSize();
    }
    
    @Override
    public void layout() {
    }
    
    public Rectangle getBounds() {
        return ((Component)this.target).getBounds();
    }
    
    @Override
    public boolean isFocusable() {
        return false;
    }
    
    @Override
    public GraphicsConfiguration getGraphicsConfiguration() {
        if (this.winGraphicsConfig != null) {
            return this.winGraphicsConfig;
        }
        return ((Component)this.target).getGraphicsConfiguration();
    }
    
    public SurfaceData getSurfaceData() {
        return this.surfaceData;
    }
    
    public void replaceSurfaceData() {
        this.replaceSurfaceData(this.numBackBuffers, this.backBufferCaps);
    }
    
    public void createScreenSurface(boolean b) {
        this.surfaceData = ScreenUpdateManager.getInstance().createScreenSurface((Win32GraphicsConfig)this.getGraphicsConfiguration(), this, this.numBackBuffers, b);
    }
    
    public void replaceSurfaceData(int numBackBuffers, BufferCapabilities backBufferCaps) {
        SurfaceData surfaceData = null;
        VolatileImage backBuffer = null;
        synchronized (((Component)this.target).getTreeLock()) {
            synchronized (this) {
                if (this.pData == 0L) {
                    return;
                }
                this.numBackBuffers = numBackBuffers;
                ScreenUpdateManager instance = ScreenUpdateManager.getInstance();
                surfaceData = this.surfaceData;
                instance.dropScreenSurface(surfaceData);
                this.createScreenSurface(true);
                if (surfaceData != null) {
                    surfaceData.invalidate();
                }
                backBuffer = this.backBuffer;
                if (this.numBackBuffers > 0) {
                    this.backBufferCaps = backBufferCaps;
                    this.backBuffer = ((Win32GraphicsConfig)this.getGraphicsConfiguration()).createBackBuffer(this);
                }
                else if (this.backBuffer != null) {
                    this.backBufferCaps = null;
                    this.backBuffer = null;
                }
            }
        }
        if (surfaceData != null) {
            surfaceData.flush();
        }
        if (backBuffer != null) {
            backBuffer.flush();
        }
    }
    
    public void replaceSurfaceDataLater() {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                if (!WComponentPeer.this.isDisposed()) {
                    try {
                        WComponentPeer.this.replaceSurfaceData();
                    }
                    catch (InvalidPipeException ex) {}
                }
            }
        };
        Component component = (Component)this.target;
        if (!PaintEventDispatcher.getPaintEventDispatcher().queueSurfaceDataReplacing(component, runnable)) {
            this.postEvent(new InvocationEvent(component, runnable));
        }
    }
    
    @Override
    public boolean updateGraphicsData(GraphicsConfiguration graphicsConfiguration) {
        this.winGraphicsConfig = (Win32GraphicsConfig)graphicsConfiguration;
        try {
            this.replaceSurfaceData();
        }
        catch (InvalidPipeException ex) {}
        return false;
    }
    
    @Override
    public ColorModel getColorModel() {
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
        if (graphicsConfiguration != null) {
            return graphicsConfiguration.getColorModel();
        }
        return null;
    }
    
    public ColorModel getDeviceColorModel() {
        Win32GraphicsConfig win32GraphicsConfig = (Win32GraphicsConfig)this.getGraphicsConfiguration();
        if (win32GraphicsConfig != null) {
            return win32GraphicsConfig.getDeviceColorModel();
        }
        return null;
    }
    
    public ColorModel getColorModel(int n) {
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
        if (graphicsConfiguration != null) {
            return graphicsConfiguration.getColorModel(n);
        }
        return null;
    }
    
    @Override
    public Graphics getGraphics() {
        if (this.isDisposed()) {
            return null;
        }
        Component parent = (Component)this.getTarget();
        Window containingWindow = SunToolkit.getContainingWindow(parent);
        if (containingWindow != null) {
            Graphics translucentGraphics = ((WWindowPeer)containingWindow.getPeer()).getTranslucentGraphics();
            if (translucentGraphics != null) {
                int n = 0;
                int n2 = 0;
                for (; parent != containingWindow; parent = parent.getParent()) {
                    n += parent.getX();
                    n2 += parent.getY();
                }
                translucentGraphics.translate(n, n2);
                translucentGraphics.clipRect(0, 0, parent.getWidth(), parent.getHeight());
                return translucentGraphics;
            }
        }
        SurfaceData surfaceData = this.surfaceData;
        if (surfaceData != null) {
            Color color = this.background;
            if (color == null) {
                color = SystemColor.window;
            }
            Color color2 = this.foreground;
            if (color2 == null) {
                color2 = SystemColor.windowText;
            }
            Font font = this.font;
            if (font == null) {
                font = WComponentPeer.defaultFont;
            }
            return ScreenUpdateManager.getInstance().createGraphics(surfaceData, this, color2, color, font);
        }
        else {
            return null;
        }
    }
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        return WFontMetrics.getFontMetrics(font);
    }
    
    private synchronized native void _dispose();
    
    @Override
    protected void disposeImpl() {
        SurfaceData surfaceData = this.surfaceData;
        this.surfaceData = null;
        ScreenUpdateManager.getInstance().dropScreenSurface(surfaceData);
        surfaceData.invalidate();
        WToolkit.targetDisposedPeer(this.target, this);
        this._dispose();
    }
    
    public void disposeLater() {
        this.postEvent(new InvocationEvent(this.target, new Runnable() {
            @Override
            public void run() {
                WComponentPeer.this.dispose();
            }
        }));
    }
    
    @Override
    public synchronized void setForeground(Color foreground) {
        this.foreground = foreground;
        this._setForeground(foreground.getRGB());
    }
    
    @Override
    public synchronized void setBackground(Color background) {
        this.background = background;
        this._setBackground(background.getRGB());
    }
    
    public Color getBackgroundNoSync() {
        return this.background;
    }
    
    private native void _setForeground(int p0);
    
    private native void _setBackground(int p0);
    
    @Override
    public synchronized void setFont(Font font) {
        this._setFont(this.font = font);
    }
    
    synchronized native void _setFont(Font p0);
    
    @Override
    public void updateCursorImmediately() {
        WGlobalCursorManager.getCursorManager().updateCursorImmediately();
    }
    
    public boolean requestFocus(Component component, boolean temporary, boolean focusedWindowChangeAllowed, long time, CausedFocusEvent.Cause cause) {
        if (KeyboardFocusManagerPeerImpl.processSynchronousLightweightTransfer((Component)this.target, component, temporary, focusedWindowChangeAllowed, time)) {
            return true;
        }
        switch (WKeyboardFocusManagerPeer.shouldNativelyFocusHeavyweight((Component)this.target, component, temporary, focusedWindowChangeAllowed, time, cause)) {
            case 0: {
                return false;
            }
            case 2: {
                if (WComponentPeer.focusLog.isLoggable(PlatformLogger.Level.FINER)) {
                    WComponentPeer.focusLog.finer("Proceeding with request to " + component + " in " + this.target);
                }
                Window containingWindow = SunToolkit.getContainingWindow((Component)this.target);
                if (containingWindow == null) {
                    return this.rejectFocusRequestHelper("WARNING: Parent window is null");
                }
                WWindowPeer wWindowPeer = (WWindowPeer)containingWindow.getPeer();
                if (wWindowPeer == null) {
                    return this.rejectFocusRequestHelper("WARNING: Parent window's peer is null");
                }
                int requestWindowFocus = wWindowPeer.requestWindowFocus(cause);
                if (WComponentPeer.focusLog.isLoggable(PlatformLogger.Level.FINER)) {
                    WComponentPeer.focusLog.finer("Requested window focus: " + (boolean)requestWindowFocus);
                }
                if (requestWindowFocus == 0 || !containingWindow.isFocused()) {
                    return this.rejectFocusRequestHelper("Waiting for asynchronous processing of the request");
                }
                return WKeyboardFocusManagerPeer.deliverFocus(component, (Component)this.target, temporary, focusedWindowChangeAllowed, time, cause);
            }
            case 1: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    
    private boolean rejectFocusRequestHelper(String msg) {
        if (WComponentPeer.focusLog.isLoggable(PlatformLogger.Level.FINER)) {
            WComponentPeer.focusLog.finer(msg);
        }
        KeyboardFocusManagerPeerImpl.removeLastFocusRequest((Component)this.target);
        return false;
    }
    
    public Image createImage(ImageProducer is) {
        return new ToolkitImage(is);
    }
    
    @Override
    public Image createImage(int n, int n2) {
        return ((Win32GraphicsConfig)this.getGraphicsConfiguration()).createAcceleratedImage((Component)this.target, n, n2);
    }
    
    @Override
    public VolatileImage createVolatileImage(int width, int height) {
        return new SunVolatileImage((Component)this.target, width, height);
    }
    
    public boolean prepareImage(Image image, int n, int n2, ImageObserver imageObserver) {
        return Toolkit.getDefaultToolkit().prepareImage(image, n, n2, imageObserver);
    }
    
    public int checkImage(Image image, int n, int n2, ImageObserver imageObserver) {
        return Toolkit.getDefaultToolkit().checkImage(image, n, n2, imageObserver);
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[" + this.target + "]";
    }
    
    WComponentPeer(Component target) {
        this.isLayouting = false;
        this.paintPending = false;
        this.oldWidth = -1;
        this.oldHeight = -1;
        this.numBackBuffers = 0;
        this.backBuffer = null;
        this.backBufferCaps = null;
        this.serialNum = 0;
        this.isAccelCapable = true;
        this.target = target;
        this.paintArea = new RepaintArea();
        this.create(this.getNativeParent());
        this.checkCreation();
        this.createScreenSurface(false);
        this.initialize();
        this.start();
    }
    
    abstract void create(WComponentPeer p0);
    
    WComponentPeer getNativeParent() {
        return (WComponentPeer)WToolkit.targetToPeer(SunToolkit.getNativeContainer((Component)this.target));
    }
    
    protected void checkCreation() {
        if (this.hwnd == 0L || this.pData == 0L) {
            if (this.createError != null) {
                throw this.createError;
            }
            throw new InternalError("couldn't create component peer");
        }
        else {
            return;
        }
    }
    
    synchronized native void start();
    
    void initialize() {
        if (((Component)this.target).isVisible()) {
            this.show();
        }
        Color foreground = ((Component)this.target).getForeground();
        if (foreground != null) {
            this.setForeground(foreground);
        }
        Font font = ((Component)this.target).getFont();
        if (font != null) {
            this.setFont(font);
        }
        if (!((Component)this.target).isEnabled()) {
            this.disable();
        }
        Rectangle bounds = ((Component)this.target).getBounds();
        this.setBounds(bounds.x, bounds.y, bounds.width, bounds.height, 3);
    }
    
    void handleRepaint(int n, int n2, int n3, int n4) {
    }
    
    void handleExpose(int n, int n2, int n3, int n4) {
        this.postPaintIfNecessary(n, n2, n3, n4);
    }
    
    public void handlePaint(int n, int n2, int n3, int n4) {
        this.postPaintIfNecessary(n, n2, n3, n4);
    }
    
    private void postPaintIfNecessary(int x, int y, int w, int h) {
        if (!AWTAccessor.getComponentAccessor().getIgnoreRepaint((Component)this.target)) {
            PaintEvent paintEvent = PaintEventDispatcher.getPaintEventDispatcher().createPaintEvent((Component)this.target, x, y, w, h);
            if (paintEvent != null) {
                this.postEvent(paintEvent);
            }
        }
    }
    
    void postEvent(AWTEvent event) {
        this.preprocessPostEvent(event);
        SunToolkit.postEvent(SunToolkit.targetToAppContext(this.target), event);
    }
    
    void preprocessPostEvent(AWTEvent awtEvent) {
    }
    
    public void beginLayout() {
        this.isLayouting = true;
    }
    
    public void endLayout() {
        if (!this.paintArea.isEmpty() && !this.paintPending && !((Component)this.target).getIgnoreRepaint()) {
            this.postEvent(new PaintEvent((Component)this.target, 800, new Rectangle()));
        }
        this.isLayouting = false;
    }
    
    public native void beginValidate();
    
    public native void endValidate();
    
    public Dimension preferredSize() {
        return this.getPreferredSize();
    }
    
    @Override
    public synchronized void addDropTarget(DropTarget dropTarget) {
        if (this.nDropTargets == 0) {
            this.nativeDropTargetContext = this.addNativeDropTarget();
        }
        ++this.nDropTargets;
    }
    
    @Override
    public synchronized void removeDropTarget(DropTarget dropTarget) {
        --this.nDropTargets;
        if (this.nDropTargets == 0) {
            this.removeNativeDropTarget();
            this.nativeDropTargetContext = 0L;
        }
    }
    
    native long addNativeDropTarget();
    
    native void removeNativeDropTarget();
    
    native boolean nativeHandlesWheelScrolling();
    
    @Override
    public boolean handlesWheelScrolling() {
        return this.nativeHandlesWheelScrolling();
    }
    
    public boolean isPaintPending() {
        return this.paintPending && this.isLayouting;
    }
    
    @Override
    public void createBuffers(int n, BufferCapabilities bufferCapabilities) throws AWTException {
        ((Win32GraphicsConfig)this.getGraphicsConfiguration()).assertOperationSupported((Component)this.target, n, bufferCapabilities);
        try {
            this.replaceSurfaceData(n - 1, bufferCapabilities);
        }
        catch (InvalidPipeException ex) {
            throw new AWTException(ex.getMessage());
        }
    }
    
    @Override
    public void destroyBuffers() {
        this.replaceSurfaceData(0, null);
    }
    
    @Override
    public void flip(int n, int n2, int n3, int n4, BufferCapabilities.FlipContents flipContents) {
        VolatileImage backBuffer = this.backBuffer;
        if (backBuffer == null) {
            throw new IllegalStateException("Buffers have not been created");
        }
        ((Win32GraphicsConfig)this.getGraphicsConfiguration()).flip(this, (Component)this.target, backBuffer, n, n2, n3, n4, flipContents);
        return;
    }
    
    @Override
    public synchronized Image getBackBuffer() {
        VolatileImage backBuffer = this.backBuffer;
        if (backBuffer == null) {
            throw new IllegalStateException("Buffers have not been created");
        }
        return backBuffer;
    }
    
    public BufferCapabilities getBackBufferCaps() {
        return this.backBufferCaps;
    }
    
    public int getBackBuffersNum() {
        return this.numBackBuffers;
    }
    
    public boolean shouldClearRectBeforePaint() {
        return true;
    }
    
    native void pSetParent(ComponentPeer p0);
    
    @Override
    public void reparent(ContainerPeer containerPeer) {
        this.pSetParent(containerPeer);
    }
    
    @Override
    public boolean isReparentSupported() {
        return true;
    }
    
    public void setBoundsOperation(int n) {
    }
    
    public boolean isAccelCapable() {
        return this.isAccelCapable && isContainingTopLevelAccelCapable((Component)this.target) && (!SunToolkit.isContainingTopLevelTranslucent((Component)this.target) || Win32GraphicsEnvironment.isVistaOS());
    }
    
    public void disableAcceleration() {
        this.isAccelCapable = false;
    }
    
    native void setRectangularShape(int p0, int p1, int p2, int p3, Region p4);
    
    private static final boolean isContainingTopLevelAccelCapable(Component parent) {
        while (parent != null && !(parent instanceof WEmbeddedFrame)) {
            parent = parent.getParent();
        }
        return parent == null || ((WEmbeddedFramePeer)parent.getPeer()).isAccelCapable();
    }
    
    @Override
    public void applyShape(Region obj) {
        if (WComponentPeer.shapeLog.isLoggable(PlatformLogger.Level.FINER)) {
            WComponentPeer.shapeLog.finer("*** INFO: Setting shape: PEER: " + this + "; TARGET: " + this.target + "; SHAPE: " + obj);
        }
        if (obj != null) {
            this.setRectangularShape(obj.getLoX(), obj.getLoY(), obj.getHiX(), obj.getHiY(), obj.isRectangular() ? null : obj);
        }
        else {
            this.setRectangularShape(0, 0, 0, 0, null);
        }
    }
    
    @Override
    public void setZOrder(ComponentPeer componentPeer) {
        this.setZOrder((componentPeer != null) ? ((WComponentPeer)componentPeer).getHWnd() : 0L);
    }
    
    private native void setZOrder(long p0);
    
    public boolean isLightweightFramePeer() {
        return false;
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.windows.WComponentPeer");
        shapeLog = PlatformLogger.getLogger("sun.awt.windows.shape.WComponentPeer");
        focusLog = PlatformLogger.getLogger("sun.awt.windows.focus.WComponentPeer");
        defaultFont = new Font("Dialog", 0, 12);
    }
}
